import pygame
from enum import Enum, auto
from typing import List, Tuple

class Direction(Enum):
    UP = auto()
    DOWN = auto()
    LEFT = auto()
    RIGHT = auto()

class Snake:
    def __init__(self, start_pos: Tuple[int, int], block_size: int):
        self.block_size = block_size
        self.direction = Direction.RIGHT
        self.body = [start_pos]  # Head is at index 0
        self.growing = False
        
        # Movement cooldown
        self.move_cooldown = 100  # milliseconds
        self.last_move_time = 0

    def move(self, current_time: int) -> bool:
        """Move the snake if enough time has passed. Returns True if moved."""
        if current_time - self.last_move_time < self.move_cooldown:
            return False

        # Update position
        head = self.body[0]
        new_head = self._get_new_head_position(head)
        
        # Insert new head
        self.body.insert(0, new_head)
        
        # Remove tail if not growing
        if not self.growing:
            self.body.pop()
        else:
            self.growing = False
        
        self.last_move_time = current_time
        return True

    def _get_new_head_position(self, head: Tuple[int, int]) -> Tuple[int, int]:
        x, y = head
        if self.direction == Direction.UP:
            return (x, y - self.block_size)
        elif self.direction == Direction.DOWN:
            return (x, y + self.block_size)
        elif self.direction == Direction.LEFT:
            return (x - self.block_size, y)
        else:  # Direction.RIGHT
            return (x + self.block_size, y)

    def grow(self):
        """Mark the snake to grow on next move"""
        self.growing = True

    def check_collision(self, width: int, height: int) -> bool:
        """Check if snake has collided with walls or itself"""
        head = self.body[0]
        
        # Check wall collision
        if (head[0] < 0 or head[0] >= width or 
            head[1] < 0 or head[1] >= height):
            return True
        
        # Check self collision (skip head)
        if head in self.body[1:]:
            return True
        
        return False

    def change_direction(self, new_direction: Direction):
        """Change direction if it's not opposite to current direction"""
        opposites = {
            Direction.UP: Direction.DOWN,
            Direction.DOWN: Direction.UP,
            Direction.LEFT: Direction.RIGHT,
            Direction.RIGHT: Direction.LEFT
        }
        
        if opposites[new_direction] != self.direction:
            self.direction = new_direction

    def draw(self, screen: pygame.Surface):
        """Draw the snake on the screen"""
        for segment in self.body:
            pygame.draw.rect(
                screen,
                (0, 255, 0),  # Green color
                pygame.Rect(segment[0], segment[1], self.block_size, self.block_size)
            ) 